<?php
/**
 * GiftPay - Payment Module for PrestaShop 1.7
 *
 * This file is the declaration of the module.
 *
 * @author GiftPay
 * @license https://opensource.org/licenses/afl-3.0.php
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class GiftPay extends PaymentModule
{
    public $address;

    /**
     * PrestaPay constructor.
     *
     * Set the information about this module
     */
    public function __construct()
    {
        $this->name                   = 'giftpay';
        $this->tab                    = 'payments_gateways';
        $this->version                = '1.0';
        $this->author                 = 'GiftPay';
        $this->controllers            = array('payment', 'validation');
        $this->currencies             = true;
        $this->currencies_mode        = 'checkbox';
        $this->bootstrap              = true;
        $this->displayName            = 'GiftPay';
        $this->description            = 'Accept payments via retail gift cards or cryptocurrency.';
        $this->confirmUninstall       = 'Are you sure you want to uninstall this module?';
        $this->ps_versions_compliancy = array('min' => '1.7.0', 'max' => _PS_VERSION_);

        parent::__construct();
    }

    const PS_OS_GP_AWAITING_PAYMENT_COLOR = '#C0C0C0';
    const PS_OS_GP_AWAITING_FULL_PAYMENT_COLOR = '#2F4F4F';

    /**
     * Install this module and register the following Hooks:
     *
     * @return bool
     */
    public function install()
    {
        $this->addOrderState('GIftPay - awaiting payment', 'PS_OS_GP_AWAITING_PAYMENT', self::PS_OS_GP_AWAITING_PAYMENT_COLOR);
        $this->addOrderState('GIftPay - awaiting full payment', 'PS_OS_GP_AWAITING_FULL_PAYMENT', self::PS_OS_GP_AWAITING_FULL_PAYMENT_COLOR);

        Configuration::updateValue('PS_WEBSERVICE', 1);

        return parent::install()
            && $this->registerHook('paymentOptions')
            && $this->registerHook('paymentReturn');
    }

    /**
     * Uninstall this module and remove it from all hooks
     *
     * @return bool
     */
    public function uninstall()
    {
        return parent::uninstall();
    }

    /**
     * Display this module as a payment option during the checkout
     *
     * @param array $params
     * @return array|void
     */
    public function hookPaymentOptions($params)
    {
        /*
         * Verify if this module is active
         */
        if (!$this->active) {
            return;
        }

        /**
         * Form action URL. The form data will be sent to the
         * validation controller when the user finishes
         * the order process.
         */
        $formAction = $this->context->link->getModuleLink($this->name, 'validation', array(), true);

        /**
         * Assign the url form action to the template var $action
         */
        $this->smarty->assign(['action' => $formAction]);

        /**
         *  Load form template to be displayed in the checkout step
         */
        $paymentForm = $this->fetch('module:giftpay/views/templates/hook/payment_options.tpl');

        /**
         * Create a PaymentOption object containing the necessary data
         * to display this module in the checkout
         */
        $newOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption;
        $newOption->setModuleName($this->displayName)
            ->setCallToActionText($this->displayName)
            ->setAction($formAction)
            ->setForm($paymentForm);

        $payment_options = array(
            $newOption
        );

        return $payment_options;
    }

    /**
     * Display a message in the paymentReturn hook
     *
     * @param array $params
     * @return string
     */
    public function hookPaymentReturn($params)
    {
        /**
         * Verify if this module is enabled
         */
        if (!$this->active) {
            return;
        }

        return $this->fetch('module:giftpay/views/templates/hook/payment_return.tpl');
    }

    private $configurations = [
        'MERCHANTS_CODE',
        'SECRET_TOKEN',
        'SANDBOX_MODE',
        'SANDBOX_MERCHANTS_CODE',
        'SANDBOX_SECRET_TOKEN'
    ];

    public function getContent()
    {
        $output = null;

        if (Tools::isSubmit('submit'.$this->name)) {
            $values = Tools::getAllValues();

            foreach ($this->configurations as $configuration) {
                $value = $values[$configuration];

                if (isset($values[$configuration])) {
                    if (empty($value) && $configuration != 'SANDBOX_MODE') {
                        $output .= $this->displayError($this->l('Invalid ' . $configuration));
                    } elseif ($value != Configuration::get($configuration)){
                        Configuration::updateValue($configuration, $value);
                        $output .= $this->displayConfirmation($this->l($configuration . '  updated'));
                    }
                }
            }
        }

        $result = $output.$this->displayForm();

        return $result.'<p>You must add the webhook url to your <a href="https://giftpay.me/account/settings/websites" 
            target="_blank">Business Account</a>. 
            This will enable you to receive notifications on the charge statuses. '
            . _PS_BASE_URL_ . '/index.php?fc=module&module=' . $this->name . '&controller=webhook</p>';
    }

    public function displayForm()
    {
        // Get default language
        $defaultLang = (int)Configuration::get('PS_LANG_DEFAULT');

        // Init Fields form array
        $fieldsForm[0]['form'] = [
            'legend' => [
                'title' => $this->l('Settings'),
            ],
            'input' => [
                [
                    'type' => 'text',
                    'label' => $this->l('Merchant\'s Code'),
                    'name' => 'MERCHANTS_CODE',
                    'size' => 20,
                    'required' => true
                ],
                [
                    'type' => 'text',
                    'label' => $this->l('Secret token'),
                    'name' => 'SECRET_TOKEN',
                    'size' => 20,
                    'required' => true
                ],
                [
                    'type' => 'switch',
                    'label' => $this->l('Sandbox mode'),
                    'name' => 'SANDBOX_MODE',
                    'is_bool' => true,
                    'required' => false,
                    'values' => [
                        [
                            'id' => 'sandbox_mode_on',
                            'value' => 1,
                            'label' => $this->l('Enabled')
                        ],
                        [
                            'id' => 'sandbox_mode_off',
                            'value' => 0,
                            'label' => $this->l('Disabled')
                        ]
                    ]
                ],
                [
                    'type' => 'text',
                    'label' => $this->l('Sandbox merchant\'s code'),
                    'name' => 'SANDBOX_MERCHANTS_CODE',
                    'size' => 20,
                    'required' => false
                ],
                [
                    'type' => 'text',
                    'label' => $this->l('Sandbox secret token'),
                    'name' => 'SANDBOX_SECRET_TOKEN',
                    'size' => 20,
                    'required' => false
                ]
            ],
            'submit' => [
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right'
            ]
        ];

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        // Language
        $helper->default_form_language = $defaultLang;
        $helper->allow_employee_form_lang = $defaultLang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;        // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submit'.$this->name;
        $helper->toolbar_btn = [
            'save' => [
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
                    '&token='.Tools::getAdminTokenLite('AdminModules'),
            ],
            'back' => [
                'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            ]
        ];

        // Load current value
        $helper->fields_value['MERCHANTS_CODE'] = Tools::getValue('MERCHANTS_CODE', Configuration::get('MERCHANTS_CODE'));
        $helper->fields_value['SECRET_TOKEN'] = Tools::getValue('SECRET_TOKEN', Configuration::get('SECRET_TOKEN'));
        $helper->fields_value['SANDBOX_MODE'] = Tools::getValue('SANDBOX_MODE', Configuration::get('SANDBOX_MODE'));
        $helper->fields_value['SANDBOX_MERCHANTS_CODE'] = Tools::getValue('SANDBOX_MERCHANTS_CODE', Configuration::get('SANDBOX_MERCHANTS_CODE'));
        $helper->fields_value['SANDBOX_SECRET_TOKEN'] = Tools::getValue('SANDBOX_SECRET_TOKEN', Configuration::get('SANDBOX_SECRET_TOKEN'));

        return $helper->generateForm($fieldsForm);
    }

    private function addOrderState($name, $configName, $color)
    {
        $state_exist = false;
        $states = OrderState::getOrderStates((int)$this->context->language->id);

        // check if order state exist
        foreach ($states as $state) {
            if (in_array($name, $state)) {
                $state_exist = true;
                break;
            }
        }

        // If the state does not exist, we create it.
        if (!$state_exist) {
            // create new order state
            $order_state = new OrderState();
            $order_state->color = $color;
            $order_state->send_email = false;
//            $order_state->template = 'name of your email template';
            $order_state->module_name = $this->name;
            $order_state->name = [];
            $order_state->logable = false;
            $languages = Language::getLanguages(false);
            foreach ($languages as $language)
                $order_state->name[ $language['id_lang'] ] = $name;

            // Update object
            $order_state->add();

            // Update if not exists create state config
            Configuration::updateValue($configName, $order_state->id);
        }

        return true;
    }
}
